#!/usr/bin/perl -w -T

### BEGIN USER CONFIGURED FILES :

$template_file = "/path/to/template/search.html";
$log_file      = "/path/to/optional/log/search.log";

# Verbosity Levels
# ================
# 0  = None (no logging)
# 5  = Log errors only
# 10 = Log errors and searches
# 20 = Log errors, searches and some runtime information 
#      (useful for cursory debugging)
# 30 = Log *everything*.  (useful for in-depth debugging)

$verbosity     = 0;

### END USER CONFIGURATION (Don't edit anything below this line)

# The user should only need to modify the three variables above.

#10 Dec 99 - JDP - Added calendar back in but it's commented by default.
#10 Dec 99 - JDP - "Previous" and "Next Page" can appear in 
#                  "results_head" and / or "results_tail".
#10 Dec 99 - JDP - fixed parse swish-e parse problem.
#10 Dec 99 - JDP - added new type "slink" so you can use can have 
#                  "hot related sites!"
#02 Dec 99 - JDP - added verbosity levels for the log file.
#25 Nov 99 - SRE - skip preview (and error msg) if file path is a URL 
#                  (in swish_head)
#25 Nov 99 - SRE - added $file_not_found to template, NOTFOUND arg to this 
#                  script (for use as "404 file not found" replacement via 
#                  .htaccess in Apache)
#22 Nov 99 - SRE - replaced backticks with capture_stdout() routine 
#                  for security
#22 Nov 99 - SRE - added "taint" flag, renamed $swish variable to $searchexe 
#                  to match $searchcgi
#19 Nov 99 - SRE - display error messages like "err: all search words too 
#                  common to be useful"
#19 Nov 99 - SRE - change "Previous" and "Next Page" display when there ARE 
#                  no other pages
#19 Nov 99 - SRE - trap either "# results:" or "# Number of hits:" for 
#                  match count total
#18 Nov 99 - SRE - added ARGV and file_not_found to supply "file missing" 
#                  text in front of search box
#13 Nov 99 - SRE - removed calendar stuff, now don't need Mysql
#11 Nov 99 - SRE - added $swishiadmin to template, was undefined variable 
#                  before!
# 2 Nov 99 - SRE - added template option to keep search box at top of 
#                  results page
# 2 Nov 99 - SRE - added log file and replaced exit() with log_exit()... but
#                  still always exit status 0 ???
# 2 Nov 99 - SRE - added new type (swish++), previous only choice (swish-e) 
#                  uses different command line
# 1 Nov 99 - SRE - rename 'head' to 'swish_head' to avoid redefinition, 
#                  Mysql module missing, etc
#11 Jan 99 - JDP - added README, and some minor changes.
#14 Feb 98 - JDP - uses "templates", and handles previews for different
#                  file formats (html, pdf, txt)
#17 Nov 97 - JDP - better abstractions.
#11 Nov 97 - JDP - rewrote "head" bits in perl.  Uses GET instead of POST.
#Spring 97 - JDP - first release.  simplistic interface to Swish.

use CGI qw(:standard);

$ENV{"PATH"} = "/bin"; 

# Create log file and use it instead of STDERR.

if ($verbosity > 0)
  {
    if (defined ($log_file))
      {
	if (!open (LOGFILE, ">> $log_file")) 
	  {
	    print STDERR "5:Can't open $log_file for appended output.\n";
	    exit (0);
	  }
	$tmp = scalar localtime;
	log_it (20, "Begin:$tmp.");
      }
  }

## BEGINING of template processing:

# Read the template.
$template = "";

if (!open (INPUT, "< $template_file")) 
  {
    log_it (5, "Can't open $template_file as input.");
    log_exit (0);
  }
else 
  {
    while (<INPUT>) 
      { 
        $cnt++;
	$template .= $_; 
      } 
    close (INPUT);
  }

log_it (20, "Read $cnt lines from $template_file");

# Parse the template for categories.
$cnt = 0;
while ($template =~ m/<!--\s+index\s/) 
  {
    $cnt++;
    $template =~ s/(\<\!\--\s+index\s+)(\S+)(\s+)(\S+)(\s+)(\S+)(\s+)(\S.*)(\s+-->)//i;
    push @category, $8;
    push @category_type, $2;
    push @category_file, $6;
    push @category_total, 0;
    $_ = $4;
    if (m/UNSELECTED/i)
      {
	push @category_selection, 0; # false (0) == UNSELECTED
      }
    else
      {
	push @category_selection, 1; # true (1) == SELECTED
      }
  }

log_it (20, "Found $cnt categories in $template_file");

# Parse the template for replacement rules.
while ($template =~ m/<!--\s+replacement-rules\s/) 
  {
    $template =~ s/(\<\!\--\s+replacement-rules\s+)(\S+)(\s+)(\S+)(\s+-->)//i;
    push @replace, $2;
    push @replacement, $4;
  }
$cnt = scalar @replace;

log_it (20, "Found $cnt replacement paths in $template_file");

# Parse the template for page size (number of results per page)
if ($template =~ m/(<!--\s+page-size\s+)(\d+)(\s+-->)/i)
  {
    $template =~ s/(<!--\s+page-size\s+)(\d+)(\s+-->)//i;
    $page_size = $2;
  }
else  
  {
    $page_size = 5;
  }
  
# Parse the template for the maximum number of matches to find.
if ($template =~ m/(<!--\s+maximum-matches\s+)(\d+)(\s+-->)/i)
  {
    $template =~ s/(<!--\s+maximum-matches\s+)(\d+)(\s+-->)//i;
    $maximum_matches = $2;
  }
else
  {
    $maximum_matches = 100;
  }

# Parse the template for preview read size (how big a file chunk to read for preview)
if ($template =~ m/<!--\s+preview-read-size\s+\d+\s+-->/i)
  {
    $template =~ s/(<!--\s+preview-read-size\s+)(\d+)(\s+-->)//i;
    $preview_read_size = $2;
  }
else
  {
    $preview_read_size = 2000;
  }

# Parse the template for preview size (how long the preview should be)
if ($template =~ m/<!--\s+preview-size\s+\d+\s+-->/i)
  {
    $template =~ s/(<!--\s+preview-size\s+)(\d+)(\s+-->)//i;
    $preview_size = $2;
  }
else
  {
    $preview_size = 320;
  }

# Parse the template for the path to this script.
my ($searchcgi, $searchexe);

if ($template =~ m/(<!--\s+search-cgi\s+)(\S+)(\s+-->)/i)
  {
    $template =~ s/(<!--\s+search-cgi\s+)(\S+)(\s+-->)//i;
    $searchcgi = $2;
  }
else
  {
    $searchcgi = "search.cgi";
  }

log_it (20, "Path back to this search script: " . $searchcgi);

# Parse the template for the swish command.
if ($template =~ m/(<!--\s+swish-command\s+)(\S+)(\s+-->)/i)
  {
    $template =~ s/(<!--\s+swish-command\s+)(\S+)(\s+-->)//i;
    $searchexe = $2;
  }
else 
  {
    $searchexe = "/usr/local/bin/swish-e";
  }

log_it (20, "Path to the search engine: " . $searchexe);

# The template is divided into several sections described in the README;
# the next regular expressions splice the template into these sections.

$template =~ s/<!--\s+start of document head\s+-->.*?<!--\s+end of document head\s+-->//s;
$document_head = $&;
$template =~ s/<!-- start of document tail -->.*?<!-- end of document tail -->//s;
$document_tail = $&;

# "file not found" section for .htaccess redirection added 11/99 - SRE
$template =~ s/<!-- start of file not found -->.*?<!-- end of file not found -->//s;
$file_not_found = $&;

$template =~ s/<!-- start of search summary head -->.*?<!-- end of search summary head -->//s;
$search_summary_head = $&;
$template =~ s/<!-- start of search summary tail -->.*?<!-- end of search summary tail -->//s;
$search_summary_tail = $&;
$template =~ s/<!-- start of search summary body -->.*?<!-- end of search summary body -->//s;
$search_summary_body = $&;

$template =~ s/<!-- start of search results head -->.*?<!-- end of search results head -->//s;
$search_results_head = $&;
$template =~ s/<!-- start of search results html -->.*?<!-- end of search results html -->//s;
$search_results_html = $&;
$template =~ s/<!-- start of search results pdf -->.*?<!-- end of search results pdf -->//s;
$search_results_pdf = $&;
$template =~ s/<!-- start of search results slink -->.*?<!-- end of search results slink -->//s;
$search_results_slink = $&;
$template =~ s/<!-- start of search results calendar -->.*?<!-- end of search results calendar -->//s;
$search_results_calendar = $&;
$template =~ s/<!-- start of search results tail -->.*?<!-- end of search results tail -->//s;
$search_results_tail = $&;

$template =~ s/<!-- start of search box head -->.*?<!-- end of search box head -->//s;
$search_box_head = $&;
$template =~ s/<!-- start of search box tail -->.*?<!-- end of search box tail -->//s;
$search_box_tail = $&;
$template =~ s/<!-- start of search box body1 -->.*?<!-- end of search box body1 -->//s;
$search_box_body_A = $&;
$template =~ s/<!-- start of search box body -->.*?<!-- end of search box body -->//s;
$search_box_body_B = $&;

# Parse the template for where to put search box in results page
if ($template =~ m/<!--\s+search_box_location\s+\S+\s+-->/i)
  {
    $template =~ s/(<!--\s+search_box_location\s+)([a-z]+)(\s+-->)//i;
    $search_box_location = $2;
  }
else
  {
    $search_box_location = "bottom";
  }

# Parse the template for how to notify site administrator
if ($template =~ m/<!--\s+swishiadmin\s.*-->/i)
  {
    $template =~ s/(<!--\s+swishiadmin\s+)(.+?)(\s+-->)//i;
    $swishiadmin = $2;
  }
else
  {
    $search_box_location = "bottom";
  }

## END of template processing.

print header;
print $document_head;

if (defined($ARGV[0])) # Use this script as ErrorDocument in Apache .htaccess
  { 
    print $file_not_found if (defined ($file_not_found) and ($ARGV[0] eq "NOTFOUND") ); 
  }
    

## BEGINING of search results processing

if (param ()) 
  {
    $i=0;
    @scope = param ("scope");
    foreach $category_entry (@category) 
      {
	$category_selection[$i] = 0; # unselected
	foreach $scope_entry (@scope) 
	  {
	    if ($category_entry eq $scope_entry) 
	      { 
		$category_selection[$i] = 1; # selected
	      }
	  }
	$i++;
      }
    
    if (param("start")) 
      {
	$start = param("start");
	$end = param("end");
      } 
    else 
      {
	$start = 1;
	$end = $page_size;
      }
    
    $words = param ("words");
    $words =~ s/\W/ /g;
    print_search_box () if ($search_box_location eq "top");
    
    if (@category) 
      {
	my ($file, @searchopt, $return_value, 
	    $results_complete, $results_partial, 
	    $results_partial_heading, $results_partial_tail);
	
	print $search_summary_head;
	
	$i=0;
	foreach $category_entry (@category) 
	  {
	    if ($category_selection[$i] == 1) 
	      {
		$results_partial = "";
		
		## swish++ search wrapper
		if ($category_type[$i] =~ m/swish\+\+/i) 
		  {
		    # 11/99 - SRE - swish++ can return ONLY the lines you want, 
		    #               so we don't need to walk thru all of them
		    # -mmax-results   The maximum number of results to return
		    # -rskip-results  The initial number of results to skip. The default is 0.
		    $tmp = $start - 1;
		    @searchopt = ( "-s", "-m$maximum_matches", "-r$tmp" );
		    @tmp = split /,/, $category_file[$i];
		    push @searchopt, "-i";
		    push @searchopt, @tmp;
		    @tmp = split /\s+/, $words;
		    push @searchopt, @tmp;
		    log_it (10, "swish++:$searchexe @searchopt");
		    $return_value = capture_stdout($searchexe, @searchopt);
		    $results_partial .= &swish_parse($return_value, $start, $end, "plus");
		    $category_total[$i] = $total;
		  }
		
		## swish-e search wrapper
		if ($category_type[$i] =~ m/swish\-e/i) 
		  {
		    # NOTE: -m is NOT the same as number of results to DISPLAY per page!
		    @searchopt = ( "-m", "$maximum_matches", "-f" );
		    @tmp = split /,/, $category_file[$i];
		    push @searchopt, @tmp;
		    push @searchopt, "-w";
		    @tmp = split /\s+/, $words;
		    push @searchopt, @tmp;
		    log_it (10, "swish-e:$searchexe @searchopt"); 
		    $return_value = capture_stdout($searchexe, @searchopt);
		    $results_partial .= &swish_parse($return_value, $start, $end, "e");
		    $category_total[$i] = $total;
		  }
		
		if ($category_type[$i] =~ m/calendar/i) 
		  {
		    $mgout .= &process_calendar($category_file[$i], $words, $start, $end);
		    $category_total[$i] = $total;
		  }

		# Glue the results together and figure out what the next and 
		# previous pages are.

                $results_partial_tail = $search_results_tail;
		
		$results_partial_heading = $search_results_head;
		if ($start > 1) 
		  {
		    $newstart = $start - $page_size;
		    $newend = $newstart + $page_size - 1;
		    $previous_page = "<a href=\"$searchcgi\?start=$newstart" . 
		      "&end=$newend&words=" . &url_escape("$words") . 
			"&scope=" . &url_escape("$category_entry") . "\">Previous Page</a>";
		  }
		else
		  { 
		    $previous_page = "(at first page)";
		  } 
		  
		$results_partial_heading =~ s/Previous Page/$previous_page/;
		$results_partial_tail =~ s/Previous Page/$previous_page/;
		
		if ($end < $category_total[$i]) 
		  {
		    $newstart = $start + $page_size;
		    $newend = $newstart + $page_size - 1;
		    $next_page = "<a href=\"$searchcgi\?start=$newstart" .
		      "&end=$newend&words=" . &url_escape("$words") . 
			"&scope=" . &url_escape("$category_entry") . "\">Next Page</a>";
		  }
		else
		  { 
		    $next_page = "(at last page)";
		  } 
		  
		$results_partial_heading =~ s/Next Page/$next_page/;
		$results_partial_heading =~ s/Entry Title/$category_entry/g;
		$results_partial_tail =~ s/Next Page/$next_page/;
		$results_partial_tail =~ s/Entry Title/$category_entry/g;
		
		if ($category_total[$i] != 0) 
		  {
		    $results_complete = $results_complete . $results_partial_heading . 
		                        $results_partial . $results_partial_tail;
		  }
	      }
	    $i++;
	  }
	
	$i=0;
	foreach $category_entry (@category) 
	  {
	    if ($category_selection[$i] == 1) 
	      {
		$results_search_summary = $search_summary_body;
		if ($category_total[$i] == 0) 
		  {
		    $results_search_summary =~ s/Entry Title/$category_entry/g;
		    $results_search_summary =~ s/Entry Range//;
		  } 
		else 
		  {
		    $results_search_summary =~ s/Entry Title/<a href\=\"\#$category_entry\">$category_entry<\/a>/g;
		    if ($start <= $category_total[$i]) 
		      {
			if ($end > $category_total[$i]) 
			  {
			    $results_search_summary =~ s/Entry Range/$start - $category_total[$i]/;
			  } 
			else 
			  {
			    $results_search_summary =~ s/Entry Range/$start - $end/;
			  }
		      }
		  }
		$results_search_summary =~ s/Entry Total/$category_total[$i] Total/;
		print $results_search_summary;
	      }
	    $i++;
	  }
	print $search_summary_tail;
	print $results_complete;
      }
  }
else # no params --> may not have printed search box
  { 
    print_search_box () if($search_box_location eq "top"); 
  }


print_search_box () if ($search_box_location eq "bottom");
print $document_tail;

log_exit(0);

### END OF MAIN PROCEDURE


### Print the search box.

sub print_search_box
  {
    if (!defined ($words)) { $words = ""; }
    $search_box_head =~ s/DEFAULT\_VALUE/$words/;
    print $search_box_head;
    $i=0;
    foreach $category_entry (@category) 
      {
	if ($i==0) 
	  {
	    $search_box = $search_box_body_A;
	  } 
	else 
	  {
	    $search_box = $search_box_body_B;
	  }
	if ($category_selection[$i] == 0) 
	  {  
	    $search_box =~ s/CHECKED//i;
	  }
	$search_box =~ s/Entry Title/$category_entry/g;
	print $search_box;
	$i++;
      }
    print $search_box_tail;
  }

### close log_file and exit

sub log_exit
  {
    $tmp = scalar localtime;
    log_it (20, "End:status=$_[0], $tmp");
    
    if ($verbosity > 0)
      {
	close(LOGFILE) 
	  if (defined ($log_file));
      }
    
    exit($_[0]);
  }

sub log_it
  {
    my($level, $string);
    $level = $_[0];
    $string = $_[1];
    
    if ($verbosity >= $level)
      {
	$string =~ s/\n/\n$level\:/g;
	print LOGFILE "$level:$string\n"
	  if (defined ($log_file)); 
      }
    
    # If logging isn't on, then send the errors (level 5) to STDERR
    if (($verbosity == 0) || (!defined ($log_file)))
      {
	print STDERR "$level:$string\n"
	  if ($level == 5);
      }
  }

### Use the replacement rules to manipulate the links.

sub link_replacement 
  {
    my($link, $replace_this, $with_this, $i);
    $link = $_[0];
    
    log_it (30, "replacing LINK: \"$link\"");
    
    $i=0;
    foreach $replace_this (@replace) 
      {
	$with_this = $replacement[$i];
	log_it (30, "replacing FROM: \"$replace_this\"");
	log_it (30, "replacing TO: \"$with_this\"");
	$link =~ s/$replace_this/$with_this/i;
	$i++;
      }
    
    log_it (30, " FINAL LINK: \"$link\"\n");
    
    return($link);
  }

### Create a result entry for an html or text file.

sub process_html 
  {
    my($link, $title, $size, $score, $preview, $rvalue, $number);
    $link  = $_[0];
    $title = $_[1];
    $size  = $_[2];
    $score = $_[3];
    $number = $_[4];
    $preview = &swish_head($link);
    
    log_it (30, "PREVIEW BELOW:\n$preview\nPREVIEW ABOVE");
    
    # Kill html tags.  Change these expressions to your preference.
    $preview =~ s|<title>.*?</title>||gsi;
    # $preview =~ s|<h2>.*?</h2>||gsi;
    # $preview =~ s|<h3>.*?</h3>||gsi;
    # $preview =~ s|<h1>.*?</h1>||gsi;
    # $preview =~ s|<i>.*?</i>||gsi;
    # $preview =~ s|<b>.*?</b>||gsi;
    $preview =~ s|< .*? >||gsx;
    $preview =~ s|\W*\n|\n|gsx;
    $preview =~ s|< .*? $||gsx;
    $preview =~ s|^.*? >||gsx;
    $preview = substr($preview, 0, $preview_size);
    
    log_it (30, "PREVIEW SUBSTR:\n$preview\nPREVIEW ABOVE\nrvalue=$rvalue"); 

    # Why don't people title their work?
    if ($title eq "") {$title="Untitled";}
    
    # Sometimes there is stray garbage in the size & score.
    $size =~ s|\D||gsx;
    $score =~ s|\D||gsx;
    
    $link = &link_replacement($link);
    
    $rvalue = $search_results_html;
    
    $rvalue =~ s/REPLACE LINK/$link/i;
    $rvalue =~ s/REPLACE TITLE/$title/i;
    $rvalue =~ s/REPLACE NUMBER/$number/i;
    $rvalue =~ s/REPLACE SIZE/$size/i;
    $rvalue =~ s/REPLACE SCORE/$score/i;
    $rvalue =~ s/REPLACE PREVIEW/$preview/i;

    log_it (30, "PREVIEW FINAL:\n$preview\nPREVIEW ABOVE\nrvalue=$rvalue");
    return ($rvalue);
  }

### Create a result entry for a PDF file.

sub process_pdf 
  {
    my($link, $title, $size, $score, $number) = @_;
    my($preview, $rvalue, $number);
    my($pdfpreview, $pdftitle, $pdfauthor, $pdfpages, $filename);

    $preview = &swish_head($link);
    
    $preview =~ s|(<pdf>)(.*?)(</pdf>)||si;
    $pdfpreview = $2;
    
    $pdfpreview =~ s|(<title>)(.*?)(</title>)||si;
    $pdftitle = $2;
    
    $pdfpreview =~ s|(<author>)(.*?)(</author>)||si;
    $pdfauthor = $2;
    
    $pdfpreview =~ s|(<pages>)(.*?)(</pages>)||si;
    $pdfpages = $2;
    
    $filename = $link;
    
    while ($filename =~ m/\//) 
      {  
	$filename =~ s/^.*?\///gsx;
      }
    $filename =~ s/\.txt$//;

    # Sometimes there is stray garbage in the size & score.
    $size =~ s|\D||gsx;
    $score =~ s|\D||gsx;
    
    # Why don't people title their work?
    if ($title eq "") 
      { 
	$title="Untitled"; 
      }

    $link = &link_replacement($link);
    $link =~ s/\.pdf\.txt$/\.pdf/i;
    
    $rvalue = $search_results_pdf;
    
    $rvalue =~ s/REPLACE LINK/$link/i;
    $rvalue =~ s/REPLACE NUMBER/$number/i;
    $rvalue =~ s/REPLACE TITLE/$pdftitle/i;
    $rvalue =~ s/REPLACE FILENAME/$filename/i;
    $rvalue =~ s/REPLACE AUTHOR/$pdfauthor/i;
    $rvalue =~ s/REPLACE PAGES/$pdfpages/i;
    $rvalue =~ s/REPLACE SIZE/$size/i;
    $rvalue =~ s/REPLACE SCORE/$score/i;
    
    return ($rvalue);
  }

# slink is another special renderer - it takes a file and uses 
# it as a link - the intent is to create a list of hot sites with
# keywords that are searched by swish-e.

sub process_slink
  {
     my($link, $title, $size, $score, $number) = @_;
     my ($i, $rvalue);
     $rvalue = "";
     if (!open(INPUT, "< $link"))
     { return ("ERROR: $link"); }
     $i=0;

     while (<INPUT>)
     {
       if ($i == 0) { $title = $_; }
       if ($i == 1) { $link = $_; }
       $i++;
     }
     close (INPUT);
     $rvalue .= $search_results_slink;
     $rvalue =~ s/REPLACE LINK/$link/i;
     $rvalue =~ s/REPLACE NUMBER/$number/i;
     $rvalue =~ s/REPLACE TITLE/$title/i;
     $rvalue =~ s/REPLACE SCORE/$score/i;
     
     return ($rvalue);
  }

# process_calendar searches a MySQL calendar of events that I 
# designed.  I have left it in (although commented) for purpose of
# example and so that I don't have to maintain multiple versions
# of this CGI.

##use Mysql;
##
##%ntomonth = 
##  (
##   "0", "All Months",
##   "1", "January",
##   "2", "February",
##   "3", "March",
##   "4", "April",
##   "5", "May",
##   "6", "June",
##   "7", "July",
##   "8", "August",
##   "9", "September",
##   "10", "October",
##   "11", "November",
##   "12", "December");
##
#### This is used exclusively for the database search.
##
##sub term_search 
##  {
##    $findme = $_[0];
##    $_ = $_[1];
##    if ($findme eq "") 
##      {
##	return(1); 
##      }
##    @terms = split /\W/, $findme;
##    foreach $term (@terms) 
##      { 
##	if (/$term/is) 
##	  { 
##	    return(1); 
##	  } 
##      }
##    return(0);
##  }
##
##sub process_calendar 
##  {
##    my ($dummy, $terms, $start, $end) = @_;
##    my ($rvalue, $calendarlink, $QueryLimit, $q, $r);
##    
##    $calendarlink = "http://agprogram.tamu.edu/calendar.cgi?county=All+Counties&month=REPLACE_MONTH&year=REPLACE_YEAR&keywords=&categorya=Research&categorya=Extension&categorya=Academic&Find=Find&.cgifields=categorya\#REPLACE_DAY";
##    $QueryLimit = 50;
##
##    if (!($dbh = Mysql->Connect ($host,"Calendar"))) 
##      { 
##	print "$Mysql::db_errstr";
##	die $Mysql::db_errstr;
##      }
##
##    $total=0;
##    $rvalue = "";
##    if ($sth = $dbh->Query ("SELECT fromdate, todate, timespan, 
##        name, description, location, address, city, state, url, contact, 
##        contactemail, contactphone, sourcename, sourceorg, sourcedept, 
##        sourcetitle, sourceemail, sourcephone, status, received, Cresearch, 
##        Cextension, Cacademic, Tfielddays, Tcountyevents, Tstateevents, 
##        Tinternalconf, Texternalconf, Tcommoditymeeting, Tlegislative
##        FROM Events ORDER BY fromdate, todate LIMIT $QueryLimit")) {
##      while (@record = $sth->FetchRow) 
##	{
##	  if (&term_search ($terms, "$record[4]" . " $record[3]") == 1) 
##	    {
##	      ($year, $month, $day) = split(/-/, $record[0], 3);
##	      $month = int($month);
##
##	      $month = $ntomonth{$month};
##	      $total++;
##	      $q = $calendarlink;
##	      $q =~ s/REPLACE_MONTH/$month/;
##	      $q =~ s/REPLACE_YEAR/$year/;
##	      $q =~ s/REPLACE_DAY/$month$day/;
##	      
##	      $r = $search_results_calendar;
##	      $r =~ s/REPLACE NUMBER/$total/;
##	      $r =~ s/REPLACE LINK/$q/;
##	      $r =~ s/REPLACE TITLE/$record[3]/;
##	      $r =~ s/REPLACE DESCRIPTION/$record[4]/;
##	      $r =~ s/REPLACE LOCATION/$record[5]/;
##	      $r =~ s/REPLACE LOCATION/$record[5]/;
##	      $r =~ s/REPLACE DATE/ $month2 $day, $year/;
##	      if (($total >= $start) && ($total <= $end)) 
##		{
##		  $rvalue .= $r;
##		}
##	    }
##	}
##    }
##    return ($rvalue);
##  }

# A dummy procedure used as a place holder.

sub process_calendar
  {
    return ("Calendar not enabled.");
  }

sub swish_parse 
  {
    my ($input,  # search results from the swish search engine
	$start,  # first search result to display (or how many results to skip!)
	$end,    # last search result to display (get $end-$start lines if possible)
	$mode ) = @_; # "plus" for swish++, "e" for swish-e (different formats!)
    my ($line, $out, $item, $error, $skip);
    my ($score, $size, $title, $link, $remainder);
    
    log_it (30, "SWISH RETURN BELOW\n$input\nSWISH RETURN ABOVE");

    $item = 0;
    $item = $start if($mode eq "plus");

    $error = 0;  # if there's an error, we don't find anything.
    $total = -1;
    
    $size = "null";
    $remainder = "null";
    $out="";
    
    # Keep reading until we've read it all.
    while ($input ne "") 
      {
	# Split off a line each time through the loop
	($line, $input) = split ('\n', $input, 2);

	log_it (30, "Processing line: $line.");
	
	$skip=1;
	
	# Don't split off any more info unless the info
	# is actually there!!!
	
	if (($line ne "") && ($line ne ".")) 
	  {
	    ($score, $link, $remainder) = split(' ', $line, 3);
	    $size = $remainder;
	    
	    if (($score ne "") || ($score ne "#"))
	      {
                if($mode eq "plus") # swish++ has different syntax
                  {
		    ($size, $title) = split(' ', $remainder, 2);
	          }
                else
                  {
		    ($title, $size) = split('" ', $remainder, 2);
		    $title =~ s|\"||osx;
	          }
	      }
	    else 
	      {
		$skip=0;
	      }

	    # do NOT trap error if we are simply unable to find the requested words
	    if (($score eq "err:") && ($error == 0) && ($line !~ /no results/))
	      {
		$error = 1;
		$out .= "Sorry, nothing was found.<br>($line)\n";
		$item++;
	      }
	  }
	else 
	  {
	    $skip=0;
	  }
	
	# If we DO have a valid entry.  Make a preview and add it to the list.
	
        # swish++ only returns the matches requested, do NOT have to walk 
	# through them all each time

	if (($mode eq "e") && (!$error) && ($score eq "#") )
	  {
	    if ($line =~ m/Number of hits/)  # swish-e line is of the form "# Number of hits: 21"
	      {
	        $line =~ m/([\d]+)/;
	        $size = int($1);
	        log_it (20, "Number of hits: $size.");
		$total = $size;

		if ($total == 0) # no matches!
		  {
		    $error = 1;
		    $out .= "Sorry, nothing was found.<br>Search string " .
		      "may be too common or not a complete word.\n";
		  }
	      }
	  }

	if (($mode eq "plus") && (!$error) && ($score eq "#") )
	  {
	    if ($link eq "results\:")  # swish++ line is of the form "# results: 21"
	      {
		$total = $size; # grab the last field, already parsed above
		if ($total == 0) # no matches!
		  {
		    $error = 1;
		    $out .= "Sorry, nothing was found.<br>Search string " .
		      "may be too common or not a complete word.\n";
		  }
	      }
	    elsif($link eq "ignored\:") # line is of the form "# ignored: search string"
	      {
		$total = 1;
		$error = 1;
		$out .= "Sorry, nothing available for find=$size<br>" .
		  "Search string may be too common or not a complete word.\n";
	      }
	    else
	      { 
		log_it (5, "While processing $line, found unexpected '#' directive.");
	      }
	  }
	
	if ((not($score eq "#")) && (not($score eq "search")) 
	    && (not($score eq "err:"))
	    && ($error == 0) && ($skip==1)) 
	  {
	    $item = $item+1 if($mode ne "plus"); # 11/99 - SRE - increment AFTER use with swish++
	    if (($item <= $end) && ($item >= $start))
	      {
		if ($link =~ /\.pdf\.txt$/i) 
		  { 
		    $out .= &process_pdf($link, $title, $size, $score, $item);
		  }
		else 
		  { 
		    if ($link =~ /\.link$/i) 
		      { 
		        $out .= &process_slink($link, $title, $size, $score, $item);
		      }
		    else
		      {
		        $out .= &process_html($link, $title, $size, $score, $item); 
		      }
		  }
	      }
	    $item = $item+1 if($mode eq "plus"); # 11/99 - SRE - increment AFTER use with swish++
	  }
      }
    $total = $item if( $total == -1 );
    
    log_it (30, "PARSED RESULTS BELOW, total=$total\n$out\nPARSED RESULTS ABOVE");

    return ($out);
  }

sub swish_head 
  {
    my($houtput);
    
    $houtput = "";

    log_it (30, "Reading preview lines from $_[0]");

    # Skip preview if the path to the file is a URL instead of a local pathname
    if($_[0] =~ m,http://,i)
      {
	$tmp = $_[0];
	$tmp =~ s,http:.*/,,; # kill the path
	$houtput = "No preview available for $tmp\n";

	log_it (30, "PREVIEW URL: $houtput");

	return ($houtput);
      }
    
    if (!open(input, "< $_[0]")) 
      {
	# You can handle this how you like - this is the case when a
	# file is not readable because of the world read permission,
	# yet it has been included in the index.  Either take it out
	# of the index or get your permissions striaghtened out!
	
	# I prefer that people just tell me about it.       
	
	$houtput = "Can't open $_[0], Please notify $swishiadmin.\n";

	log_it (5, "ERROR: $houtput");
      }
    else 
      { 
	read input, $houtput, $preview_read_size; 
        log_it (30, "PREVIEW FILE: $houtput");
      }
    close (input);
    return ($houtput);
  }

sub url_escape 
  {
    my ($temp);
    $temp = $_[0];
    $temp =~ s/\s/\+/g;
    return($temp);
  }

# The capture_stdout subroutine is used because backticks let shell 
# metacharacters evaluate program to exec should be in first arg, remaining 
# args are array of command line args
# NOTE: Can use backticks, open/exec, or spawn... first and last methods 
#       are commented out below!  (only backticks seem to work on WinNT, but 
#       are unsafe for CGI applications)

sub capture_stdout 
  {
    my $sleep_count = 0;
    my $output = "";
    my $stroke = "";
    
    # reading from child without backticks - 
    # see perlipc.html for writing example also
    do
      {
	$pid = open(KID_TO_READ, "-|");
	unless (defined $pid)
	  {
	    warn "cannot fork $_[0], attempt $sleep_count: $!";
	    die "bailing out from $_[0]" if $sleep_count++ > 6;
	    sleep 10;
	  }
      } until defined $pid;
    
    if ($pid)
      { # parent
	# @output = <KID_TO_READ>; # to create array, but we want a single string
	while (<KID_TO_READ>) { $output .= $_;  }  # capture all output from child
	$stroke = "WARNING! kid exited with status $?\n" if(!close(KID_TO_READ));
	$output = "ERROR! No output from $_[0]\n"        if(!defined($output));
	$output .= $stroke                               if( defined($tmp));
      } # end of parent processing
    else
      { # child
	# ($EUID, $EGID) = ($UID, $GID); # suid progs only
	exec(@_) || die "can't exec program $_[0]: $!"; # send output to parent
	# NOTREACHED
      } # end of child processing
    
    # example from hayes, does not work on WinNT, needs Process.pm on Unix:
    # use Process;
    # 	$task = Process->New();
    # 	$task->Spawn( sub {
    #            $test = exec($prog,@opt,@files) || die "Can't exec: $!\n";
    #            print "\nexec output follows\n$test\nend of exec output\n";
    # 	});
    
    return($output);
  }

#end of file
